@@ -4,7 +4,7 @@ from pathlib import Path
 
 import click
 
-from ravenpy.models.importers import grid_weight_importer_params
+from ravenpy.extractors.routing_product import RoutingProductGridWeightExtractor
 
 
 @click.command()
@@ -15,7 +15,7 @@ from ravenpy.models.importers import grid_weight_importer_params
     "--dim-names",
     nargs=2,
     type=click.Tuple([str, str]),
-    default=grid_weight_importer_params["DIM_NAMES"],
+    default=RoutingProductGridWeightExtractor.DIM_NAMES,
     show_default=True,
     help="Ordered dimension names of longitude (x) and latitude (y) in the NetCDF INPUT_FILE.",
 )
@@ -24,7 +24,7 @@ from ravenpy.models.importers import grid_weight_importer_params
     "--var-names",
     nargs=2,
     type=click.Tuple([str, str]),
-    default=grid_weight_importer_params["VAR_NAMES"],
+    default=RoutingProductGridWeightExtractor.VAR_NAMES,
     show_default=True,
     help="Variable name of 1D or 2D longitude and latitude variables in the NetCDF INPUT_FILE (in this order).",
 )
@@ -32,7 +32,7 @@ from ravenpy.models.importers import grid_weight_importer_params
     "-c",  # legacy script short option
     "--routing-id-field",
     type=str,
-    default=grid_weight_importer_params["ROUTING_ID_FIELD"],
+    default=RoutingProductGridWeightExtractor.ROUTING_ID_FIELD,
     show_default=True,
     help="Name of column in routing information shapefile (ROUTING_FILE) containing a unique key for each dataset.",
 )
@@ -40,7 +40,7 @@ from ravenpy.models.importers import grid_weight_importer_params
     "-f",  # legacy script short option
     "--netcdf-input-field",
     type=str,
-    default=grid_weight_importer_params["NETCDF_INPUT_FIELD"],
+    default=RoutingProductGridWeightExtractor.NETCDF_INPUT_FIELD,
     show_default=True,
     help="Attribute name in INPUT_FILE shapefile that defines the index of the shape in NetCDF model output file (numbering needs to be [0 ... N-1]).",
 )
@@ -66,7 +66,7 @@ from ravenpy.models.importers import grid_weight_importer_params
     "-e",  # legacy script short option
     "--area-error-threshold",
     type=float,
-    default=grid_weight_importer_params["AREA_ERROR_THRESHOLD"],
+    default=RoutingProductGridWeightExtractor.AREA_ERROR_THRESHOLD,
     show_default=True,
     help="Threshold (as fraction) of allowed mismatch in areas between subbasins from routing information (ROUTING_FILE) and overlay with grid-cells or subbasins (INPUT_FILE). If error is smaller than this threshold the weights will be adjusted such that they sum up to exactly 1. Raven will exit gracefully in case weights do not sum up to at least 0.95.",
 )
@@ -109,9 +109,9 @@ def generate_grid_weights(
     is then free to embed or reference, in her own config context).
     """
     # NOTE: This is in order to make sphinx-click happy. Magic. Do not touch.
-    from ravenpy.models.importers import RoutingProductGridWeightImporter
+    from ravenpy.extractors import RoutingProductGridWeightExtractor
 
-    importer = RoutingProductGridWeightImporter(
+    extractor = RoutingProductGridWeightExtractor(
         input_file,
         routing_file,
         dim_names,
@@ -122,7 +122,7 @@ def generate_grid_weights(
         sub_ids,
         area_error_threshold,
     )
-    gw_cmd = importer.extract()
+    gw_cmd = extractor.extract()
 
     if not output:
         input_file_path = Path(input_file)
