@@ -1,11 +1,11 @@
 from collections import defaultdict
-from dataclasses import dataclass
 from pathlib import Path
 
-from ravenpy.config.commands import BasinIndexCommand
-from ravenpy.config.rvs import Config
+from pydantic.dataclasses import dataclass
+
+from ravenpy.config.commands import HRU, LU, BasinIndexCommand, HRUState, Sub
+from ravenpy.config.rvs import RVI
 from ravenpy.models.base import Ostrich, Raven
-from ravenpy.models.rv import HRU, LU, RV, HRUState, Sub
 
 
 class GR4JCN(Raven):
@@ -22,8 +22,6 @@ class GR4JCN(Raven):
     doi: 10.1016/j.jhydrol.2014.04.058.
     """
 
-    identifier = "gr4jcn"
-
     @dataclass
     class Params:
         GR4J_X1: float = None
@@ -45,7 +43,6 @@ class GR4JCN(Raven):
         soil_profile: str = "DEFAULT_P"
         aquifer_profile: str = "[NONE]"
         terrain_class: str = "[NONE]"
-        _hru_type: str = "land"
 
     @dataclass
     class LakeHRU(HRU):
@@ -54,12 +51,12 @@ class GR4JCN(Raven):
         soil_profile: str = "LAKE"
         aquifer_profile: str = "[NONE]"
         terrain_class: str = "[NONE]"
-        _hru_type: str = "lake"
 
     def __init__(self, *args, **kwds):
         super().__init__(*args, **kwds)
 
-        self.config = Config(
+        self.config.update(
+            identifier="gr4jcn",
             hrus=(GR4JCN.LandHRU(),),
             subbasins=(
                 Sub(
@@ -74,7 +71,11 @@ class GR4JCN(Raven):
             derived_params=GR4JCN.DerivedParams(),
         )
 
-        self.config.rvp.tmpl = """
+        #########
+        # R V P #
+        #########
+
+        rvp_tmpl = """
         # -Global snow parameters-------------------------------------
         :RainSnowTransition 0 1.0
         :AirSnowCoeff       {derived_params.one_minus_CEMANEIGE_X2}  # [1/d] = 1.0 - CEMANEIGE_X2 = 1.0 - x6
@@ -134,20 +135,19 @@ class GR4JCN(Raven):
         # List of channel profiles
         {channel_profiles}
         """
+        self.config.rvp.set_tmpl(rvp_tmpl)
 
-        self.config.rvi.tmpl = """
-        :Calendar              {calendar}
-        :RunName               {run_name}-{run_index}
-        :StartDate             {start_date}
-        :EndDate               {end_date}
-        :TimeStep              {time_step}
-        :Method                ORDERED_SERIES
+        #########
+        # R V I #
+        #########
 
+        rvi_tmpl = """
         :SoilModel             SOIL_MULTILAYER  4
-        {routing}
+        :Routing               {routing}
         :CatchmentRoute        ROUTE_DUMP
         :Evaporation           {evaporation}  # PET_OUDIN
-        :RainSnowFraction      {rain_snow_fraction}  # RAINSNOW_DINGMAN
+        :RainSnowFraction      {rain_snow_fraction}
+
         :PotentialMeltMethod   POTMELT_DEGREE_DAY
         :OroTempCorrect        OROCORR_SIMPLELAPSE
         :OroPrecipCorrect      OROCORR_SIMPLELAPSE
@@ -180,39 +180,22 @@ class GR4JCN(Raven):
          :Flush                    RAVEN_DEFAULT      TEMP_STORE      SURFACE_WATER  			 # Qd
          :Baseflow                 BASE_GR4J          ROUTING_STORE   SURFACE_WATER  			 # Qr
         :EndHydrologicProcesses
-        #------------------------------------------------------------------------
-
-        #---------------------------------------------------------
-        # Output Options
-        #
-        :WriteForcingFunctions
-        :EvaluationMetrics {evaluation_metrics}
-        :WriteNetcdfFormat  yes
-        #:NoisyMode
-        :SilentMode
-        :PavicsMode
-        {suppress_output}
-
-        :NetCDFAttribute title Simulated river discharge
-        :NetCDFAttribute history Created on {now} by Raven
-        :NetCDFAttribute references  Craig, J.R., and the Raven Development Team, Raven user's and developer's manual (Version 2.8), URL: http://raven.uwaterloo.ca/ (2018).
-        :NetCDFAttribute comment Raven Hydrological Framework version {raven_version}
-
-        :NetCDFAttribute model_id gr4jcn
-
-        :NetCDFAttribute time_frequency day
-        :NetCDFAttribute time_coverage_start {start_date}
-        :NetCDFAttribute time_coverage_end {end_date}
         """
+        self.config.rvi.set_tmpl(rvi_tmpl)
 
-        self.config.rvi.rain_snow_fraction = "RAINSNOW_DINGMAN"
+        self.config.rvi.rain_snow_fraction = RVI.RainSnowFractionOptions.DINGMAN
         self.config.rvi.evaporation = "PET_OUDIN"
 
+        #########
+        # R V C #
+        #########
+
         # Initialize the stores to 1/2 full. Declare the parameters that can be user-modified
         self.config.rvc.soil0 = None
         self.config.rvc.soil1 = 15
 
     def derived_parameters(self):
+
         self.config.rvp.derived_params.GR4J_X1_hlf = (
             self.config.rvp.params.GR4J_X1 * 1000.0 / 2.0
         )
@@ -236,16 +219,16 @@ class GR4JCN(Raven):
             # If self.rvc.hru_states is set, it means that we are using `resume()` and we don't
             # want to interfere
             for hru in self.config.rvh.hrus:
-                if isinstance(hru, GR4JCN.LandHRU) or hru._hru_type == "land":
+                if isinstance(hru, GR4JCN.LandHRU):
                     self.config.rvc.hru_states[hru.hru_id] = HRUState(
                         index=hru.hru_id, soil0=soil0, soil1=soil1
                     )
-                elif isinstance(hru, GR4JCN.LakeHRU) or hru._hru_type == "lake":
+                elif isinstance(hru, GR4JCN.LakeHRU):
                     self.config.rvc.hru_states[hru.hru_id] = HRUState(index=hru.hru_id)
                     sb_contains_lake[hru.subbasin_id] = True
                 else:
                     raise Exception(
-                        "Type of HRU must be either `GR4JCN.LandHRU` or `GR4JCN.LakeHRU` (or its `_hru_type` must be either 'land' or 'lake')"
+                        "Type of HRU must be either `GR4JCN.LandHRU` or `GR4JCN.LakeHRU`"
                     )
 
         if not self.config.rvc.basin_states:
@@ -273,19 +256,24 @@ class GR4JCN(Raven):
 
 
 class GR4JCN_OST(Ostrich, GR4JCN):
-    _p = Path(__file__).parent / "ostrich-gr4j-cemaneige"
-    # templates = tuple(_p.glob("model/*.rv?")) + tuple(_p.glob("*.t??"))
-    templates = tuple(_p.glob("*.t??"))
-
     def __init__(self, *args, **kwds):
         super().__init__(*args, **kwds)
-        # self.config.rvi.suppress_output = True
 
-        self.config.rvc.tmpl = """
-        # Emulation of GR4J simulation of Salmon River near Prince George
-        #------------------------------------------------------------------------
-        #
-        # tied parameters:
+        self.config.update(
+            identifier="gr4jcn-ost",
+            algorithm="DDS",
+            max_iterations=50,
+            lowerBounds=GR4JCN.Params(),
+            upperBounds=GR4JCN.Params(),
+            suppress_output=True,
+        )
+
+        ####################
+        # R V C (OST TMPL) #
+        ####################
+
+        rvc_tmpl = """
+        # Tied parameters:
         # (it is important for OSTRICH to find every parameter place holder somewhere in this file)
         # (without this "para_x1" wouldn't be detectable)
         #    para_half_x1 = para_x1 / 2. = par_x1 / 2. [m] = par_half_x1 [mm]
@@ -303,9 +291,13 @@ class GR4JCN_OST(Ostrich, GR4JCN):
            1           par_half_x1   15.0
         :EndHRUStateVariableTable
         """
-        self.config.rvc.is_ostrich_tpl = True
+        self.config.rvc.set_tmpl(rvc_tmpl, is_ostrich=True)
 
-        self.config.rvp.tmpl = """
+        ####################
+        # R V P (OST TMPL) #
+        ####################
+
+        rvp_tmpl = """
         #########################################################################
         :FileType          rvp ASCII Raven 2.8.2
         :WrittenBy         Juliane Mai & James Craig
@@ -369,9 +361,13 @@ class GR4JCN_OST(Ostrich, GR4JCN):
            [DEFAULT],  par_x4,        7.73
         :EndLandUseParameterList
         """
-        self.config.rvp.is_ostrich_tpl = True
+        self.config.rvp.set_tmpl(rvp_tmpl, is_ostrich=True)
 
-        self.config.ost.tmpl = """
+        ##########
+        # O S T  #
+        ##########
+
+        ost_tmpl = """
         ProgramType         {algorithm}
         ObjectiveFunction   GCOP
         ModelExecutable     ./ostrich-runs-raven.sh
@@ -386,8 +382,8 @@ class GR4JCN_OST(Ostrich, GR4JCN):
         EndExtraDirs
 
         BeginFilePairs
-          raven-gr4j-cemaneige.rvp.tpl;    raven-gr4j-cemaneige.rvp
-          raven-gr4j-cemaneige.rvc.tpl;    raven-gr4j-cemaneige.rvc
+          {identifier}.rvp.tpl;    {identifier}.rvp
+          {identifier}.rvc.tpl;    {identifier}.rvc
           #can be multiple (.rvh, .rvi)
         EndFilePairs
 
@@ -450,15 +446,7 @@ class GR4JCN_OST(Ostrich, GR4JCN):
                 # above intializes DDS to parameter values IN the initial model input files
         EndDDSAlg
         """
-
-        self.config.update(
-            algorithm="DDS",
-            max_iterations=50,
-            lowerBounds=GR4JCN.Params(None, None, None, None, None, None),
-            upperBounds=GR4JCN.Params(None, None, None, None, None, None),
-            run_name="run",
-            run_index=0,
-        )
+        self.config.ost.set_tmpl(ost_tmpl)
 
     def derived_parameters(self):
         """Derived parameters are computed by Ostrich."""
